==> react-pose-3.3.1.tgz: package/src/components/PoseElement/index.tsx <==
import * as React from 'react';
import { isForwardRef } from 'react-is';
import { createContext, createElement } from 'react';
import poseFactory, {
  DomPopmotionPoser,
  DomPopmotionConfig
} from 'popmotion-pose';
import {
  ChildRegistration,
  CurrentPose,
  PoseContextProps,
  PoseElementInternalProps,
  PopStyle
} from './types';
import isValidProp from '@emotion/is-prop-valid';
import { invariant } from 'hey-listen';
import { hasChanged } from '../../utils/has-changed';
import { pickAssign } from '../../utils/pick-assign';

export const PoseParentContext = createContext({});

type Ref = (ref: Element) => any;
type RefSetters = {
  ref?: Ref;
  innerRef?: Ref;
  hostRef?: Ref;
};

const calcPopFromFlowStyle = (el: HTMLElement): PopStyle => {
  const { offsetTop, offsetLeft, offsetWidth, offsetHeight } = el;

  return {
    position: 'absolute',
    top: offsetTop,
    left: offsetLeft,
    width: offsetWidth,
    height: offsetHeight
  };
};

const hasPose = (pose: CurrentPose, key: string) =>
  Array.isArray(pose) ? pose.indexOf(key) !== -1 : pose === key;

const objectToMap = (obj: { [key: string]: any }): Map<string, any> =>
  Object.keys(obj).reduce((map, key) => {
    map.set(key, { raw: obj[key] });
    return map;
  }, new Map());

const testAlwaysTrue = () => true;

class PoseElement extends React.PureComponent<PoseElementInternalProps> {
  props: PoseElementInternalProps;
  poser: DomPopmotionPoser;
  ref: Element;
  styleProps: { [key: string]: any };
  shouldForwardProp: (key: string) => boolean;
  children: Set<ChildRegistration> = new Set();
  popStyle?: PopStyle;

  /**
   * Children handlers
   * =============================================
   */
  private childrenHandlers: PoseContextProps = {
    registerChild: props => {
      this.children.add(props);
      if (this.poser) this.flushChildren();
    },
    onUnmount: child => this.poser.removeChild(child),
    getParentPoseConfig: () => this.props.poseConfig,
    getInitialPoseFromParent: () => this.getInitialPose()
  };

  /**
   * Lifecycle
   * =============================================
   */
  constructor(props: PoseElementInternalProps) {
    super(props);
    this.shouldForwardProp =
      typeof this.props.elementType === 'string' ? isValidProp : testAlwaysTrue;
  }

  getInitialPose(): CurrentPose {
    const { getInitialPoseFromParent, pose, _pose, initialPose } = this.props;

    if (initialPose) {
      return initialPose;
    } else {
      // Feels like this could probably be simpler
      const parentPose = getInitialPoseFromParent && getInitialPoseFromParent();
      const thisPose = Array.isArray(pose) ? pose : [pose];
      const thisInternalPose = Array.isArray(_pose) ? _pose : [_pose];

      const initialPoses = Array.isArray(parentPose)
        ? [...parentPose, ...thisPose, ...thisInternalPose]
        : [parentPose, ...thisPose, ...thisInternalPose];

      const filteredInitialPoses = initialPoses.filter(Boolean);
      return filteredInitialPoses.length ? filteredInitialPoses : undefined;
    }
  }

  getFirstPose(): CurrentPose | void {
    const { initialPose, pose, _pose } = this.props;
    // If don't have an initial pose explicitly defined, return early
    if (!initialPose) return;

    const thisPose = Array.isArray(pose) ? pose : [pose];
    const thisInternalPose = Array.isArray(_pose) ? _pose : [_pose];

    return [...thisPose, ...thisInternalPose];
  }

  getSetProps() {
    const {
      elementType,
      poseConfig,
      onValueChange,
      innerRef,
      _pose,
      pose,
      initialPose,
      poseKey,
      onPoseComplete,
      getParentPoseConfig,
      registerChild,
      onUnmount,
      getInitialPoseFromParent,
      popFromFlow,
      values,
      parentValues,
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd,
      ...props
    } = this.props;

    // If we're popping this element out from the DOM flow, build
    // and apply position: absolute styles that visually match the previous
    // location in the DOM
    if (popFromFlow && this.ref && this.ref instanceof HTMLElement) {
      if (!this.popStyle) {
        props.style = {
          ...props.style,
          ...calcPopFromFlowStyle(this.ref as HTMLElement)
        };
        this.popStyle = props.style;
      } else {
        props.style = this.popStyle;
      }
    } else {
      this.popStyle = null;
    }

    return props;
  }

  /**
   * We need to get a ref to the underlying DOM element. Styled Components and
   * other libraries use `innerRef`, though this will be swallowed if the
   * styled component is not a primitive (ie styled(Component)).
   *
   * Instead we pass another function, `hostRef`, as recommended by Facebook
   * https://github.com/facebook/react/issues/11401
   *
   * We also only pass `ref` to DOM primitive components.
   */
  getRefs = (): RefSetters => {
    const refs: RefSetters = {};
    const { elementType } = this.props;

    if (typeof elementType === 'string' || isForwardRef(elementType)) {
      refs.ref = this.setRef;
    } else {
      refs.innerRef = this.setRef;
      refs.hostRef = this.setRef;
    }

    return refs;
  };

  setRef = (ref: Element) => {
    if (ref instanceof Element || (this.ref && ref === null)) {
      const { innerRef } = this.props;
      if (innerRef) {
        if (typeof innerRef === 'function') {
          innerRef(ref);
        } else {
          innerRef.current = ref;
        }
      }
      this.ref = ref;
    }
  };

  componentDidMount() {
    invariant(
      typeof this.ref !== 'undefined',
      `No DOM ref found. If you're converting an existing component via posed(Component), you must ensure you're passing the hostRef prop to your underlying DOM element.`
    );

    const {
      poseConfig,
      onValueChange,
      registerChild,
      values,
      parentValues,
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd
    } = this.props;

    const config: DomPopmotionConfig = {
      ...poseConfig,
      initialPose: this.getInitialPose(),
      values: values || poseConfig.values,
      parentValues: parentValues ? objectToMap(parentValues) : undefined,
      props: this.getSetProps(),
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd,
      onChange: onValueChange
    };

    // If first in tree
    if (!registerChild) {
      this.initPoser(poseFactory(this.ref, config));
    } else {
      registerChild({
        element: this.ref,
        poseConfig: config,
        onRegistered: poser => this.initPoser(poser)
      });
    }
  }

  UNSAFE_componentWillUpdate({ pose, _pose }: PoseElementInternalProps) {
    if (hasPose(pose, 'flip') || hasPose(_pose, 'flip')) this.poser.measure();
  }

  componentDidUpdate(prevProps: PoseElementInternalProps) {
    const { pose, _pose, poseKey } = this.props;
    this.poser.setProps(this.getSetProps());

    if (
      poseKey !== prevProps.poseKey ||
      hasChanged(prevProps.pose, pose) ||
      pose === 'flip'
    ) {
      this.setPose(pose);
    }

    // Internal use only. Must be a nicer way to reconcile internally and externally-set
    // poses from children.ts which uses cloneElement. Answers on a postcard.
    if (_pose !== prevProps._pose || _pose === 'flip') this.setPose(_pose);
  }

  componentWillUnmount() {
    if (!this.poser) return;
    const { onUnmount } = this.props;
    if (onUnmount) onUnmount(this.poser);
    this.poser.destroy();
  }

  initPoser(poser: DomPopmotionPoser) {
    this.poser = poser;
    this.flushChildren();

    const firstPose = this.getFirstPose();
    if (firstPose) this.setPose(firstPose);
  }

  setPose(pose: CurrentPose) {
    const { onPoseComplete } = this.props;
    const poseList: string[] = Array.isArray(pose) ? pose : [pose];

    Promise.all(poseList.map(key => key && this.poser.set(key))).then(
      () => onPoseComplete && onPoseComplete()
    );
  }

  flushChildren() {
    this.children.forEach(({ element, poseConfig, onRegistered }) =>
      onRegistered(this.poser.addChild(element, poseConfig))
    );

    this.children.clear();
  }

  render() {
    const { elementType } = this.props;

    return (
      <PoseParentContext.Provider value={this.childrenHandlers}>
        {createElement(
          elementType,
          pickAssign(this.shouldForwardProp, [
            this.getSetProps(),
            this.getRefs()
          ])
        )}
      </PoseParentContext.Provider>
    );
  }
}

export { PoseElement };

==> react-pose-3.3.6.tgz: package/src/components/PoseElement/index.tsx <==
import * as React from 'react';
//import { isForwardRef } from 'react-is';
import { createContext, createElement } from 'react';
import poseFactory, {
  DomPopmotionPoser,
  DomPopmotionConfig
} from 'popmotion-pose';
import {
  ChildRegistration,
  CurrentPose,
  PoseContextProps,
  PoseElementInternalProps,
  PopStyle
} from './types';
import isValidProp from '@emotion/is-prop-valid';
import { invariant } from 'hey-listen';
import { hasChanged } from '../../utils/has-changed';
import { pickAssign } from '../../utils/pick-assign';

export const PoseParentContext = createContext({});

type Ref = (ref: Element) => any;
type RefSetters = {
  ref?: Ref;
  innerRef?: Ref;
  hostRef?: Ref;
};

const calcPopFromFlowStyle = (el: HTMLElement): PopStyle => {
  const { offsetTop, offsetLeft, offsetWidth, offsetHeight } = el;

  return {
    position: 'absolute',
    top: offsetTop,
    left: offsetLeft,
    width: offsetWidth,
    height: offsetHeight
  };
};

const hasPose = (pose: CurrentPose, key: string) =>
  Array.isArray(pose) ? pose.indexOf(key) !== -1 : pose === key;

const objectToMap = (obj: { [key: string]: any }): Map<string, any> =>
  Object.keys(obj).reduce((map, key) => {
    map.set(key, { raw: obj[key] });
    return map;
  }, new Map());

const testAlwaysTrue = () => true;

class PoseElement extends React.PureComponent<PoseElementInternalProps> {
  props: PoseElementInternalProps;
  poser: DomPopmotionPoser;
  ref: Element;
  styleProps: { [key: string]: any };
  shouldForwardProp: (key: string) => boolean;
  children: Set<ChildRegistration> = new Set();
  popStyle?: PopStyle;

  /**
   * Children handlers
   * =============================================
   */
  private childrenHandlers: PoseContextProps = {
    registerChild: (props: ChildRegistration) => {
      this.children.add(props);
      if (this.poser) this.flushChildren();
    },
    onUnmount: (child: DomPopmotionPoser) => this.poser.removeChild(child),
    getParentPoseConfig: () => this.props.poseConfig,
    getInitialPoseFromParent: () => this.getInitialPose()
  };

  /**
   * Lifecycle
   * =============================================
   */
  constructor(props: PoseElementInternalProps) {
    super(props);
    this.shouldForwardProp =
      typeof this.props.elementType === 'string' ? isValidProp : testAlwaysTrue;
  }

  getInitialPose(): CurrentPose | void {
    const { getInitialPoseFromParent, pose, _pose, initialPose } = this.props;

    if (initialPose) {
      return initialPose;
    } else {
      const parentPose = getInitialPoseFromParent && getInitialPoseFromParent();
      const initialPoses = (Array.isArray(parentPose) ? parentPose : [parentPose])
        .concat(pose, _pose)
        .filter(Boolean);

      return initialPoses.length > 0 ? initialPoses : undefined;
    }
  }

  getFirstPose(): CurrentPose | void {
    const { initialPose, pose, _pose } = this.props;
    // If don't have an initial pose explicitly defined, return early
    if (!initialPose) return;

    const firstPose = (Array.isArray(pose) ? pose : [pose])
      .concat(_pose)
      .filter(Boolean)

    return firstPose.length === 1 ? firstPose[0] : firstPose;
  }

  getSetProps() {
    const {
      elementType,
      poseConfig,
      onValueChange,
      innerRef,
      _pose,
      pose,
      initialPose,
      poseKey,
      onPoseComplete,
      getParentPoseConfig,
      registerChild,
      onUnmount,
      getInitialPoseFromParent,
      popFromFlow,
      values,
      parentValues,
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd,
      ...props
    } = this.props;

    // If we're popping this element out from the DOM flow, build
    // and apply position: absolute styles that visually match the previous
    // location in the DOM
    if (popFromFlow && this.ref && this.ref instanceof HTMLElement) {
      if (!this.popStyle) {
        props.style = {
          ...props.style,
          ...calcPopFromFlowStyle(this.ref as HTMLElement)
        };
        this.popStyle = props.style;
      } else {
        props.style = this.popStyle;
      }
    } else {
      this.popStyle = null;
    }

    return props;
  }

  /**
   * We need to get a ref to the underlying DOM element. Styled Components and
   * other libraries use `innerRef`, though this will be swallowed if the
   * styled component is not a primitive (ie styled(Component)).
   *
   * Instead we pass another function, `hostRef`, as recommended by Facebook
   * https://github.com/facebook/react/issues/11401
   *
   * We also only pass `ref` to DOM primitive components.
   */
  getRefs = (): RefSetters => {
    const refs: RefSetters = {};
    const { elementType } = this.props;

    if (typeof elementType === 'string') {
      //  || isForwardRef(elementType) TODO: Fix types for isForwardRef
      refs.ref = this.setRef;
    } else {
      refs.innerRef = this.setRef;
      refs.hostRef = this.setRef;
    }

    return refs;
  };

  setRef = (ref: Element) => {
    if (ref instanceof Element || (this.ref && ref === null)) {
      const { innerRef } = this.props;
      if (innerRef) {
        if (typeof innerRef === 'function') {
          innerRef(ref);
        } else {
          innerRef.current = ref;
        }
      }
      this.ref = ref;
    }
  };

  componentDidMount() {
    invariant(
      typeof this.ref !== 'undefined',
      `No DOM ref found. If you're converting an existing component via posed(Component), you must ensure you're passing the hostRef prop to your underlying DOM element.`
    );

    const {
      poseConfig,
      onValueChange,
      registerChild,
      values,
      parentValues,
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd
    } = this.props;

    const config: DomPopmotionConfig = {
      ...poseConfig,
      initialPose: this.getInitialPose(),
      values: values || poseConfig.values,
      parentValues: parentValues ? objectToMap(parentValues) : undefined,
      props: this.getSetProps(),
      onDragStart,
      onDragEnd,
      onPressStart,
      onPressEnd,
      onChange: onValueChange
    };

    // If first in tree
    if (!registerChild) {
      this.initPoser(poseFactory(this.ref, config));
    } else {
      registerChild({
        element: this.ref,
        poseConfig: config,
        onRegistered: (poser: DomPopmotionPoser) => this.initPoser(poser)
      });
    }
  }

  UNSAFE_componentWillUpdate({ pose, _pose }: PoseElementInternalProps) {
    if (hasPose(pose, 'flip') || hasPose(_pose, 'flip')) this.poser.measure();
  }

  componentDidUpdate(prevProps: PoseElementInternalProps) {
    const { pose, _pose, poseKey } = this.props;
    this.poser.setProps(this.getSetProps());

    if (
      poseKey !== prevProps.poseKey ||
      hasChanged(prevProps.pose, pose) ||
      pose === 'flip'
    ) {
      this.setPose(pose);
    }

    // Internal use only. Must be a nicer way to reconcile internally and externally-set
    // poses from children.ts which uses cloneElement. Answers on a postcard.
    if (_pose !== prevProps._pose || _pose === 'flip') this.setPose(_pose);
  }

  componentWillUnmount() {
    if (!this.poser) return;
    const { onUnmount } = this.props;
    if (onUnmount) onUnmount(this.poser);
    this.poser.destroy();
  }

  initPoser(poser: DomPopmotionPoser) {
    this.poser = poser;
    this.flushChildren();

    const firstPose = this.getFirstPose();
    if (firstPose) this.setPose(firstPose);
  }

  setPose(pose: CurrentPose) {
    const { onPoseComplete } = this.props;
    const poseList: string[] = Array.isArray(pose) ? pose : [pose];

    Promise.all(poseList.map(key => key && this.poser.set(key))).then(
      () => onPoseComplete && onPoseComplete(pose)
    );
  }

  flushChildren() {
    this.children.forEach(({ element, poseConfig, onRegistered }) =>
      onRegistered(this.poser.addChild(element, poseConfig))
    );

    this.children.clear();
  }

  render() {
    const { elementType } = this.props;

    return (
      <PoseParentContext.Provider value={this.childrenHandlers}>
        {createElement(
          elementType,
          pickAssign(this.shouldForwardProp, [
            this.getSetProps(),
            this.getRefs()
          ])
        )}
      </PoseParentContext.Provider>
    );
  }
}

export { PoseElement };
